from main.core.fields import VLNFieldTracker


class TrackerMixin:
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        tracker = getattr(self.__class__, "tracker", None)
        if not tracker or not isinstance(tracker, VLNFieldTracker):
            raise AttributeError("Classes with TrackerMixin must have tracker=VLNFieldTracker()")
        self._before_save_hooks = []

    def __flush_before_save_hooks(self):
        for hook in self._before_save_hooks:
            hook()
        self._before_save_hooks.clear()

    def enqueue_before_save_hook(self, hook):
        self._before_save_hooks.append(hook)

    def save(self, *args, **kwargs):
        """
        This implementation was copy pasted from FieldTracker.patch_save
        it correctly reports tracker changes during nested model.save() calls
        """
        self.__flush_before_save_hooks()
        ret = super().save(*args, **kwargs)
        update_fields = kwargs.get("update_fields")
        if not update_fields and update_fields is not None:  # () or []
            fields = update_fields
        elif update_fields is None:
            fields = None
        else:
            fields = (field for field in update_fields if field in self.tracker.fields)
        self.tracker.set_saved_fields(fields=fields)
        return ret

    def save_if_changed(self, **kwargs):
        if not self._state.adding:
            if any(self.tracker.changed().keys()):
                self.save(**kwargs)
        else:
            self.save(**kwargs)
