import uuid

from django.contrib.auth.models import AbstractUser
from django.db import models

from users.constants import UserGroup, UserPermission


class User(AbstractUser):
    Group = UserGroup
    Permission = UserPermission

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)

    class Meta:
        verbose_name = "User"
        verbose_name_plural = "Users"
        permissions = [
            (UserPermission.Raw.IS_MANAGER, "MANAGER"),
            (UserPermission.Raw.IS_OPERATOR, "OPERATOR"),
        ]

    @property
    def full_name(self) -> str:
        if self.first_name or self.last_name:
            return " ".join([self.last_name, self.first_name])
        elif self.username:
            return self.username
        else:
            return "Unknown user"
